from pyhht.visualization import plot_imfs
import matplotlib.pyplot as plt
from scipy.io import wavfile
from scipy import signal
from pyhht import EMD
import numpy as np
import acoustics
import csv

def makeNoise():
    array = acoustics.generator.brown(1200000)
    return array

def getModel(fileName, array):
    list = []
    with open(fileName) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        for row in csv_reader:
            list.append(row)
    for i in range(len(list)):
        string = str(list[i])
        l = len(string)
        string = string[2:l-2]
        array.append(float(string))

def plot(signal, frequency, signal2 = None, signal3 = None):
    t = np.arange(0., len(signal)/frequency, 1/frequency)
    plt.plot(t,signal)
    if type(signal2) == np.ndarray:
        plt.plot(t,signal2)
    if type(signal3) == np.ndarray:
        plt.plot(t,signal3)
    plt.show()

def plot2(signal, frequency, signal2 = None, signal3 = None):
    t = np.arange(0., len(signal)/frequency, 1/frequency)
    plt.plot(t,signal)
    if type(signal2) == np.ndarray:
        plt.plot(t,signal2)
    if type(signal3) == np.ndarray:
        plt.plot(t,signal3)

def plotConv(signal, signal2, frequency):
    newSignal = np.convolve(signal,signal2)
    plot(newSignal,frequency)

def initialValues(noise):
    s,s_hat = [],[]
    getModel('h_5.csv',s),getModel('h_5.csv',s_hat)
    #s_hat = smoothing(s_hat)
    s_hat = smoothing2(s_hat)
    shortNoise = np.array(noise[0:len(s)])
    return np.array(s),np.array(s_hat),np.array(shortNoise)

def smoothing(a):
    for i in range(len(a)-3):
        a[i+1] = (a[i]+a[i+1]+a[i+2])/3
    return a

def smoothing2(a):
    a[0] = (a[0]+a[1]+a[2]+a[3]+a[4])/5
    a[1] = (a[1]+a[2]+a[3]+a[4]+a[5])/5
    a[2] = (a[2]+a[3]+a[4]+a[5]+a[6])/5
    for i in range(len(a)-5):
        a[i+2] = (a[i]+a[i+1]+a[i+2]+a[i+3]+a[i+4])/5
    return a

def inital(s_hat,x):
    return np.convolve(s_hat,x,mode='same')

def updateW(w,mu,e,x_hat):
    for i in range(len(w)-1):
        w[i] = w[i+1]
    w[-1] = w[-1] - mu*e[-1]*x_hat[-1]
    etta = 1
    if abs(w[-1]) >= etta:
        w[-1] = (etta*w[-1])/abs(w[-1])
    return w

def updateX(x,e,y_hat):
    for i in range(len(x)-1):
        x[i] = x[i+1]
    x[-1] = e[-1] - y_hat[-1]
    return x

def updateE(e,d,y_prime):
    for i in range(len(e)-1):
        e[i] = e[i+1]
    e[-1] = d[-1] + y_prime[-1]
    return e

def conv(result, first, second):
    sum, second, l, r = 0, list(reversed(second)), len(result), 100
    for i in range(l):
        if i != 0:
            result[i-1] = result[i]
    for i in range(r):
        sum = sum + (first[l-i-1]*second[l-i-1])
    result[-1] = sum
    return result

def convY(result, first, second):
    sum, second, l, r = 0, list(reversed(second)), len(result), 1
    for i in range(l):
        if i != 0:
            result[i-1] = result[i]
    for i in range(r):
        sum = sum + (first[l-i-1]*second[l-i-1])
    result[-1] = sum
    return result

def updateNoise(shortNoise,noise):
    first = noise[0]
    noise, shortNoise = np.delete(noise, 0), np.delete(shortNoise, 0)
    noise, shortNoise = np.append(noise,first),np.append(shortNoise,first)
    return shortNoise,noise

def initiate():
    noise = makeNoise()
    s, s_hat,shortNoise = initialValues(noise)
    y, y_prime, y_hat, w = np.zeros(len(s)), np.zeros(len(s)), np.zeros(len(s)), np.zeros(len(s))
    e, x = np.array(shortNoise), np.array(shortNoise)
    x_hat = inital(s_hat,x)
    XN, NN, EN, RN, SN = np.array([]), np.array([]), np.array([]), np.array([]), np.array([])
    return noise,s,s_hat,shortNoise,y,y_prime,y_hat,w,e,x,x_hat,XN,NN,EN,RN,SN

def initiateW(w,mu,e,x_hat):
    for i in range(len(w)):
        if i == 0:
            w[i] = -mu*e[i]*x_hat[i]
        else:
            w[i] = w[i-1] - mu*e[i-1]*x_hat[i-1]
    return w

def makeSpectogram(sampleFreq,noise):
    powerSpectrum, freqenciesFound, time, imageAxis = plt.specgram(noise, Fs=sampleFreq)
    plt.xlabel('Time')
    plt.ylabel('Frequency')
    plt.show()

def makePSD(frequency,noise):
    f, Pxx_den = signal.periodogram(noise, frequency)
    plt.semilogy(f, Pxx_den)
    plt.ylim([1e-10, 1e2])
    plt.xlabel('Frequency')
    plt.ylabel('PSD')
    plt.show()

def ANCprinciple(noise,fs):
    t = np.arange(0., len(noise)/fs, 1/fs)
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.plot(t,noise)
    ax.plot(t,-noise)
    ax.plot(t,noise-noise)
    plt.show()

def ANCtimedelay(noise,fs,l):
    a = np.array(-noise)
    for i in range(l):
        f = a[0]
        a = np.delete(a,0)
        a = np.append(a,f)
    t = np.arange(0., len(noise)/fs, 1/fs)
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.plot(t,noise)
    ax.plot(t,a)
    ax.plot(t,noise+a)
    plt.show()

def main():
    fs,mu = 20000, 0.25
    noise,s,s_hat,shortNoise,y,y_prime,y_hat,w,e,x,x_hat,XN,NN,EN,RN,SN = initiate()
    #makeSpectogram(fs,noise),makePSD(fs,noise),plot(noise,fs)
    plot(s,fs,s_hat)
    for i in range(10000):
        if i%10000 == 0:
            print(i)
        w = updateW(w,mu,e,x_hat)
        y = conv(y,w,x)
        y_hat = conv(y_hat,s_hat,y)
        y_prime = conv(y_prime,s,y)
        shortNoise,noise = updateNoise(shortNoise,noise)
        e = updateE(e,shortNoise,y_prime)
        x = updateX(x,e,y_hat)
        x_hat = conv(x_hat,s_hat,x)
        XN = np.append(XN,x[-1])
        NN = np.append(NN,shortNoise[-1])
        EN = np.append(EN,e[-1])
        RN = np.append(RN,y_prime[-1])
        #SN = np.append(SN,np.sign(NN[-1]))
    #plot(y_prime,fs,y_hat)
    #plot(shortNoise,fs,x)
    #plot(s,fs,s_hat)
    #plot(XN,fs,NN)
    z = np.zeros(len(XN))
    #plot(XN,fs,EN,z)
    #plot(RN,fs,z)
    #plot(noise,fs)
    #plot(shortNoise,fs,e)
    #plot(XN[len(XN)-len(shortNoise):len(XN)],fs,shortNoise)
    plt.subplot(2,1,1)
    plot2(NN,fs,EN,z)
    plt.subplot(2,1,2)
    plot2(XN,fs,NN)
    plt.show()

main()
